@@ -2,25 +2,29 @@
 
 namespace apexl\Io\includes;
 
+use apexl\Config\Singleton;
 use apexl\Io\middleware\accessRequest\accessRequest;
+use apexl\Io\middleware\addCorsHeaders;
 use apexl\Io\middleware\bodyParser\bodyParserController;
 use apexl\Io\modules\install\services\databaseTools;
-use apexl\Io\services\Output;
 use apexl\Io\modules\system\services\modules;
 use apexl\Io\services\globalData;
 use apexl\Io\services\Mailer;
+use apexl\Io\services\Output;
 use apexl\Io\services\pathUtility;
-use apexl\Config\Singleton;
 use apexl\Utils\Json\Json;
 use apexl\Vault\Vault;
-
+use DI\Container;
 use DI\ContainerBuilder;
+use Exception;
 use ForceUTF8\Encoding;
+use Psr\Container\ContainerExceptionInterface;
 use Psr\Container\ContainerInterface;
+use Psr\Container\NotFoundExceptionInterface;
+use Psr\Http\Message\ResponseInterface as Response;
+use Slim\App;
 use Slim\Exception\HttpNotFoundException;
 use Slim\Factory\AppFactory;
-use Psr\Http\Message\ResponseInterface as Response;
-use Slim\Psr7\Factory\ResponseFactory;
 use Whoops\Handler\JsonResponseHandler;
 use Zeuxisoo\Whoops\Slim\WhoopsMiddleware;
 
@@ -32,26 +36,21 @@ class System
 {
     public const HOOK__GET_VARIABLE = 'system.getVariable';
 
-    /** @var AppFactory */
+    /** @var App */
     public static $app;
 
-    /** @var \self */
+    /** @var self */
     public static $instance;
 
     /** @var modules */
     public static $module;
-
+    public static $installing = false;
     /** @var Singleton */
     protected static $config;
-
     /** @var $pathUtility */
     protected static $pathUtility;
-
     /** @var Routes */
     protected static $routes;
-
-    public static $installing = false;
-
     protected static $basePath;
 
     protected static $isInstalled = false;
@@ -59,7 +58,7 @@ class System
 
     /**
      * System constructor.
-     * @throws \Exception
+     * @throws Exception
      */
     private function __construct()
     {
@@ -69,78 +68,6 @@ class System
         $this->defineConstants();
     }
 
-    protected function defineConstants()
-    {
-        define('VERSION', self::$config->app->version ?? 1);
-    }
-
-    public function startApplication()
-    {
-        //initialise the base containers.
-        self::$module->getAvailableModules();
-
-        try {
-            $this->preFlightChecks();
-        } catch (\Exception $e) {
-            error_log("[IO Exception :: Start Application] (Line ".$e->getLine(). " | ".$e->getFile().") - ". $e->getCode()." | ".$e->getMessage());
-            //We errored on preflight. Log the error, attempt to start the base app and return.
-            Output::addSystemError($e);
-            self::$app = AppFactory::create();
-            return;
-        }
-
-        $services = self::$module->initialiseActiveModuleServices();
-
-        //Then create the app
-        AppFactory::setContainer($this->addCoreServices($services));
-        self::$app = AppFactory::create();
-        //finally loop over the modules again so we can register routes.
-        self::$module->initialiseActiveModules();
-        Routes::buildRoutes();
-    }
-
-    /**
-     * Add core services to the PHP DI container so that middleware constructors can access them
-     * @param $services
-     * @return \DI\Container
-     * @throws \Exception
-     */
-    protected function addCoreServices($services)
-    {
-        $containerBuilder = new ContainerBuilder();
-        $containerBuilder->useAnnotations(false);
-        $containerBuilder->useAutowiring(true);
-
-        //initialise core services. We array_merge so that these can be overridden by modules if needed.
-        $coreServices = [
-            pathUtility::class => function () {
-                return pathUtility::getInstance();
-            },
-            Singleton::class => function () {
-                return Singleton::getInstance();
-            },
-            Vault::class => function () {
-                return Vault::getInstance();
-            },
-            Mailer::class => function () {
-                return new Mailer();
-            },
-            globalData::class => function () {
-                return globalData::getInstance();
-            },
-            Output::class => function () {
-                return Output::getInstance();
-            }
-        ];
-
-        //Register services declared by modules.
-        $services = array_merge($coreServices, $services);
-        $containerBuilder->addDefinitions($services);
-
-        //Build PHP DI Containers
-        return $containerBuilder->build();
-    }
-
     /**
      * Allows us to keep the system class as a singleton.
      * @return System
@@ -150,78 +77,13 @@ class System
         if (!self::$instance) {
             self::$instance = new System();
         }
+
         return self::$instance;
     }
 
-    /**
-     * Function to perform a number of initial checks prior to launching.
-     */
-    protected function preFlightChecks()
+    protected function defineConstants()
     {
-        //check if we have the required config set:
-        self::testAvailableConfig();
-
-        //check if we have vault config. If not we need to install.
-        $requireInstall = $this::$config->app->installRequired ?? true;
-        if (!$this->isInstalled() && $requireInstall) {
-            $installPath = self::getInstallPath();
-            if (self::$pathUtility->getPath(0) != $installPath) {
-                Output::addResponse(null, ['redirect' => System::getInstallPath()], false);
-                Output::addMessage('requires_install', "error", "This Application requires installation. Please set {app: {installRequired: FALSE}} in config, or install the application. Install Path: " . $installPath);
-            }
-
-            //force the loading of any provided install display module.
-            if (self::$config->app->install->display ?? false) {
-                self::$module->setModuleFirst(self::$config->app->install->display);
-            }
-
-            //add the installer to the modules list.
-            self::$module->setModuleFirst("install");
-            self::$installing = true;
-        }
-    }
-    /**
-     * Method to check if the System is installed, or if it needs to go through the install process.
-     */
-    public function isInstalled()
-    {
-        if (self::$installing) {
-            return false;
-        }
-        if (self::$isInstalled) {
-            return true;
-        }
-        //first check if the database exists:
-        self::$isInstalled = (new databaseTools())->checkDBExists();
-        //next check if a core table exists
-        if (self::$isInstalled) {
-            try {
-                $database = Vault::getInstance();
-                if ($database->isInitialised()) {
-                    $row = $database->select('sites')->fields()->limit(1)->execute()->fetchAssoc();
-                } else {
-                    self::$isInstalled = false;
-                }
-            } catch (\Exception $e) {
-                error_log("[IO Exception :: Is Installed Check] (Line ".$e->getLine(). " | ".$e->getFile().")  - ". $e->getCode()." | ".$e->getMessage());
-                self::$isInstalled = false; //not installed.
-            }
-        }
-        return self::$isInstalled;
-    }
-
-    private function getInstaller()
-    {
-        return isset(self::$config->app->installer) && !empty(self::$config->app->installer) ? self::$config->app->installer : "\apexl\Io\modules\install\installModule";
-    }
-
-    /**
-     * This function allows for a custom root install path to be specified. It is also possible to provide custom install methods.
-     * @return string
-     */
-    public static function getInstallPath()
-    {
-        return isset(self::$config->app->install->path) && !empty(self::$config->app->install->path) ? self::$config->app->install->path : "install";
+        define('VERSION', self::$config->app->version ?? 1);
     }
 
     public static function switchDatabase($name)
@@ -232,13 +94,14 @@ class System
 
     /**
      * Load and decode a json schema file.
-     * @todo add checks, defaults etc.
      * @param $schemaPath
      * @return mixed
+     * @todo add checks, defaults etc.
      */
     public static function loadSchema($schemaPath)
     {
         $schema = file_get_contents($schemaPath);
+
         return json_decode($schema);
     }
 
@@ -255,43 +118,7 @@ class System
 
         $whoopsHandlers = Hook::processHook('whoopsHandlers', [new JsonResponseHandler()]);
         self::$app->add(new WhoopsMiddleware(['enable' => true], $whoopsHandlers));
-
-        //deal with cors:
-        self::$app->options('{routes:.+}', function ($request, $response, $args) {
-            return $response;
-        });
-
-
-        self::$app->add(function ($request, $response) {
-            try {
-                $response = $response->handle($request);
-            } catch (\Exception $e) {
-                error_log("[IO Exception :: Response] (Line ".$e->getLine(). " | ".$e->getFile().") - ". $e->getCode()." | ".$e->getMessage());
-                Hook::processHook('error', $e);
-                Output::addSystemError($e);
-                $response = (new ResponseFactory())->createResponse();
-                return System::asJson($response, [], 500);
-            }
-            //load the config and get the referer
-            $config = Singleton::getInstance();
-            $referer = $request->getHeader("HTTP_REFERER");
-            $referer = array_shift($referer);
-            if ($config->app->environment == 'development' && is_null($referer)) {
-                //assume Postman for now, so allow it.
-                return self::addCorsHeaders($response, "*");
-            }
-
-            //get the host and check it exists in the allowed domains.
-            $parsedUrl = parse_url($referer);
-
-            //if it does, set the headers. if not, just return response.
-            if (in_array($parsedUrl['scheme'].'://'.$parsedUrl['host'], $config->app->allowedDomains)) {
-                $domain = $parsedUrl['scheme'].'://'.$parsedUrl['host'];
-                $domain .= isset($parsedUrl['port']) ? ':'.$parsedUrl['port'] : '';
-                return self::addCorsHeaders($response, $domain);
-            }
-            return $response;
-        });
+        self::$app->add(addCorsHeaders::class);
 
         /**
          * Catch-all route to serve a 404 Not Found page if none of the routes match
@@ -300,18 +127,22 @@ class System
         self::$app->map(['GET', 'POST', 'PUT', 'DELETE', 'PATCH'], '/{routes:.+}', function ($request, $response) {
             try {
                 throw new HttpNotFoundException($request);
-            } catch (\Exception $e) {
-                $fullUrl = (empty($_SERVER['HTTPS']) ? 'http' : 'https') . "://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]";
-                error_log("[IO Exception :: Routing] (Line ".$e->getLine(). " | ".$e->getFile().") - ". $e->getCode()." | ".$e->getMessage()." | URL:".$fullUrl);
+            } catch (Exception $e) {
+                error_log(
+                    "[IO Exception :: Routing] (Line ".$e->getLine()." | ".$e->getFile().") - ".$e->getCode(
+                    )." | ".$e->getMessage()
+                );
                 //did we thorw a true 404? or is there an exisitng system error?
                 if (!Output::getSystemErrors()) {
                     //no errors? throw the 404, otherwise return the system error code..
                     Output::addSystemError($e, '404');
+
                     return System::asJson($response, [], 404);
                 } else {
                     //we got atleast one system error. return the code for the first one we got.
                     $existingErrors = Output::getSystemErrors() ?? [];
                     $error = array_shift($existingErrors);
+
                     return System::asJson($response, [], ($error['status'] ?? 500));
                 }
             }
@@ -320,29 +151,29 @@ class System
         self::$app->run();
     }
 
-    /**
-     * Function to add CORS as needed on the fly.
-     * @param $response
-     * @param string $domain
-     * @return mixed
-     */
-    public static function addCorsHeaders($response, $domain = "*")
+    public static function asJson(Response $response, $data = [], $status = 200): Response
     {
-        return $response
-            ->withHeader('Access-Control-Allow-Origin', $domain)
-            ->withHeader('Access-Control-Allow-Headers', 'X-Requested-With, Content-Type, Accept, Origin, Authorization')
-            ->withHeader('Access-Control-Allow-Methods', 'GET, POST, PUT, DELETE, PATCH, OPTIONS')
-            ->withHeader('Access-Control-Allow-Credentials', 'true')
+        //grab the messages class, or the any overridden version of it.
+        $output = self::getRegisteredService(Output::class) ?? Output::getInstance();
+        $data = array_merge($data, $output::getOutput());
+        $response->getBody()->write(json_encode($data));
 
-            // Cache Headers
-            ->withHeader('Cache-Control', 'no-cache, no-store')
-            ;
+        return $response
+            ->withHeader('Content-Type', 'application/json')
+            ->withStatus($status);
     }
 
-    public static function redirect($location, $status = 302)
+    /**
+     * Get actual service by base service FQDN (allows us to collect overridden services)
+     * @throws ContainerExceptionInterface
+     * @throws NotFoundExceptionInterface
+     */
+    public static function getRegisteredService($service): mixed
     {
-        header('Location: ' . $location, true, $status);
-        exit();
+        /** @var ?ContainerInterface $container */
+        $container = self::$app->getContainer();
+
+        return $container?->get($service);
     }
 
     public static function get($name)
@@ -350,42 +181,28 @@ class System
         return self::$app->$name;
     }
 
+    public static function redirect($location, $status = 302): never
+    {
+        header('Location: '.$location, true, $status);
+        exit();
+    }
+
     public static function writeConfig($key, $config)
     {
         Singleton::updateConfigFileByKey($key, $config);
         self::$config->reloadConfig();
     }
 
-    public static function asJson(Response $response, $data = [], $status = 200): Response
-    {
-        //grab the messages class, or the any overridden version of it.
-        $output = self::getRegisteredService(Output::class) ?? Output::getInstance();
-        $data = array_merge($data, $output::getOutput());
-        $response->getBody()->write(json_encode($data));
-        return $response
-            ->withHeader('Content-Type', 'application/json')
-            ->withStatus($status);
-    }
-
     /**
-     * Get actual service by base service FQDN (allows us to collect overridden services)
-     * @param $service
-     * @return mixed
+     * @throws ContainerExceptionInterface
+     * @throws NotFoundExceptionInterface
      */
-    public static function getRegisteredService($service)
+    public static function makeRegisteredService(string $service, array $params = []): mixed
     {
-        /** @var ContainerInterface $container */
+        /** @var ?ContainerInterface $container */
         $container = self::$app->getContainer();
 
-        return $container ? $container->get($service) : null;
-    }
-
-    public static function makeRegisteredService(string $service, array $params = [])
-    {
-        /** @var ContainerInterface $container */
-        $container = self::$app->getContainer();
-
-        return $container ? $container->make($service, $params) : null;
+        return $container?->make($service, $params);
     }
 
     public static function output($messages, $route = '', $type = 'redirect'): array
@@ -415,10 +232,10 @@ class System
         $messagesData = [];
         if ($content = $messages['messages'] ?? false) {
             $messagesData = $content;
-        } elseif ($content =  $messages['content'] ?? false) {
+        } elseif ($content = $messages['content'] ?? false) {
             $messagesData[] = [
                 'content' => $messages['content'],
-                'type' => $success ? 'success' : 'error'
+                'type' => $success ? 'success' : 'error',
             ];
         } else {
             $messagesData = null;
@@ -429,6 +246,7 @@ class System
         if ($event) {
             $output[$event] = $route;
         }
+
         return $output;
     }
 
@@ -438,12 +256,8 @@ class System
         foreach ($data as $key => $value) {
             $object->$key = Encoding::fixUTF8($value);
         }
-        return $object;
-    }
 
-    public static function setBasePath($basePath)
-    {
-        self::$basePath = $basePath;
+        return $object;
     }
 
     public static function getBasePath()
@@ -451,6 +265,11 @@ class System
         return self::$basePath;
     }
 
+    public static function setBasePath($basePath)
+    {
+        self::$basePath = $basePath;
+    }
+
     public static function moduleIsInstalled($module): bool
     {
         //if we're installing the system then return false. Otherwise we can check in our system table.
@@ -458,100 +277,41 @@ class System
         return !self::$installing;
     }
 
-    protected static function testAvailableConfig()
-    {
-        $missingConfig = [];
-        $requiredAppConfig = [
-            "app" => [
-                "environment",
-                "allowedDomains"
-            ]
-        ];
-        //TODO allow other modules to specify required configuration.
-        //@TODO allow for other methods to easily test for any missing config, required or not.
-        foreach ($requiredAppConfig as $key => $configItem) {
-            if (is_array($configItem)) {
-                foreach ($configItem as $subItem) {
-                    if (!self::isConfigSet($key, $subItem)) {
-                        $missingConfig[] = $key.': '.$subItem;
-                    }
-                }
-            } elseif (!self::isConfigSet($key, $configItem)) {
-                $missingConfig[] = $key.': '.$configItem;
-            }
-        }
-
-        if ($missingConfig !== []) {
-            throw new \Exception("Required application config is missing: (".implode(',', $missingConfig).")");
-        }
-    }
-
-    protected static function isConfigSet($key, $item)
-    {
-        return (isset(self::$config->$key->$item) && !empty(self::$config->$key->$item)) ?? false;
-    }
-
-    public static function getModuleByName($name)
-    {
-        $vault = Vault::getInstance();
-        return $vault->select('modules')->fields()->where('name', $name)->fetchAssoc()->execute();
-    }
-
     public static function getInstalledModuleByName($name)
     {
         $module = self::getModuleByName($name);
         if ($module['installed'] ?? false) {
             return $module;
         }
+
         return false;
     }
 
-    public static function hasVariable(string $name, int $site = 1): bool
+    public static function getModuleByName($name)
     {
-        //DB isn't available when it's not installed, so skip the query if we're running core install.
-        if (System::$isInstalled) {
-            $vault = Vault::getInstance();
+        $vault = Vault::getInstance();
 
-            $var = $vault->
-                select('variables')
-                ->fields('count(*) as count')
-                ->where('site', $site)
-                ->where('name', $name)
-                ->execute()
-                ->fetchAssoc();
-
-            return (int) $var['count'] !== 0;
-        }
-
-        return false;
+        return $vault->select('modules')->fields()->where('name', $name)->fetchAssoc()->execute();
     }
 
     /**
      * @param $name
      * @param $data
-     * @param int $site
+     * @param  int  $site
      * @return mixed
      */
     public static function setVariable(string $name, $data, int $site = 1)
     {
         $vault = Vault::getInstance();
-        //first, detect the data type.
-
-        switch (gettype($data)) {
-            case 'array':
-            case 'object':
-            case 'boolean':
-                $cleanedData = json_encode($data);
-                break;
-            default:
-                $cleanedData = $data;
-                break;
-        }
+        $cleanedData = match (gettype($data)) {
+            'array', 'object', 'boolean' => json_encode($data),
+            default => $data,
+        };
         //insert or update?
         if (self::hasVariable($name, $site)) {
             $vault->update('variables')->fields([
                 'value' => $cleanedData,
-                'site' => $site
+                'site' => $site,
             ])->where('site', $site)
                 ->where('name', $name)
                 ->execute();
@@ -562,12 +322,32 @@ class System
         $vault->insert('variables')->fields([
             'name' => $name,
             'value' => $cleanedData,
-            'site' => $site
+            'site' => $site,
         ])->execute();
 
         return $data;
     }
 
+    public static function hasVariable(string $name, int $site = 1): bool
+    {
+        //DB isn't available when it's not installed, so skip the query if we're running core install.
+        if (System::$isInstalled) {
+            $vault = Vault::getInstance();
+
+            $var = $vault->
+            select('variables')
+                ->fields('count(*) as count')
+                ->where('site', $site)
+                ->where('name', $name)
+                ->execute()
+                ->fetchAssoc();
+
+            return (int) $var['count'] !== 0;
+        }
+
+        return false;
+    }
+
     /**
      * @return mixed
      */
@@ -612,6 +392,7 @@ class System
                 }
             }
         }
+
         return $vars;
     }
 
@@ -625,4 +406,198 @@ class System
         $vault = Vault::getInstance();
         $vault->delete('variables')->fields()->where('site', $site)->where('name', $name)->execute();
     }
+
+    public static function isCli(): bool
+    {
+        static $isCli;
+        if (!isset($isCli)) {
+            $isCli = php_sapi_name() === 'cli';
+        }
+
+        return $isCli;
+    }
+
+    public function startApplication()
+    {
+        //initialise the base containers.
+        self::$module->getAvailableModules();
+
+        try {
+            $this->preFlightChecks();
+        } catch (Exception $e) {
+            error_log(
+                "[IO Exception :: Start Application] (Line ".$e->getLine()." | ".$e->getFile().") - ".$e->getCode(
+                )." | ".$e->getMessage()
+            );
+            //We errored on preflight. Log the error, attempt to start the base app and return.
+            Output::addSystemError($e);
+            self::$app = AppFactory::create();
+
+            return;
+        }
+
+        $services = self::$module->initialiseActiveModuleServices();
+
+        //Then create the app
+        AppFactory::setContainer($this->addCoreServices($services));
+        self::$app = AppFactory::create();
+        //finally loop over the modules again so we can register routes.
+        self::$module->initialiseActiveModules();
+        Routes::buildRoutes();
+    }
+
+    /**
+     * Function to perform a number of initial checks prior to launching.
+     */
+    protected function preFlightChecks()
+    {
+        //check if we have the required config set:
+        self::testAvailableConfig();
+
+        //check if we have vault config. If not we need to install.
+        $requireInstall = $this::$config->app->installRequired ?? true;
+        if (!$this->isInstalled() && $requireInstall) {
+            $installPath = self::getInstallPath();
+            if (self::$pathUtility->getPath(0) != $installPath) {
+                Output::addResponse(null, ['redirect' => System::getInstallPath()], false);
+                Output::addMessage(
+                    'requires_install',
+                    "error",
+                    "This Application requires installation. Please set {app: {installRequired: FALSE}} in config, or install the application. Install Path: ".$installPath
+                );
+            }
+
+            //force the loading of any provided install display module.
+            if (self::$config->app->install->display ?? false) {
+                self::$module->setModuleFirst(self::$config->app->install->display);
+            }
+
+            //add the installer to the modules list.
+            self::$module->setModuleFirst("install");
+            self::$installing = true;
+        }
+    }
+
+    protected static function testAvailableConfig()
+    {
+        $missingConfig = [];
+        $requiredAppConfig = [
+            "app" => [
+                "environment",
+                "allowedDomains",
+            ],
+        ];
+        //TODO allow other modules to specify required configuration.
+        //@TODO allow for other methods to easily test for any missing config, required or not.
+        foreach ($requiredAppConfig as $key => $configItem) {
+            if (is_array($configItem)) {
+                foreach ($configItem as $subItem) {
+                    if (!self::isConfigSet($key, $subItem)) {
+                        $missingConfig[] = $key.': '.$subItem;
+                    }
+                }
+            } elseif (!self::isConfigSet($key, $configItem)) {
+                $missingConfig[] = $key.': '.$configItem;
+            }
+        }
+
+        if ($missingConfig !== []) {
+            throw new Exception("Required application config is missing: (".implode(',', $missingConfig).")");
+        }
+    }
+
+    protected static function isConfigSet($key, $item)
+    {
+        return (isset(self::$config->$key->$item) && !empty(self::$config->$key->$item)) ?? false;
+    }
+
+    /**
+     * Method to check if the System is installed, or if it needs to go through the install process.
+     */
+    public function isInstalled()
+    {
+        if (self::$installing) {
+            return false;
+        }
+        if (self::$isInstalled) {
+            return true;
+        }
+        //first check if the database exists:
+        self::$isInstalled = (new databaseTools())->checkDBExists();
+        //next check if a core table exists
+        if (self::$isInstalled) {
+            try {
+                $database = Vault::getInstance();
+                if ($database->isInitialised()) {
+                    $row = $database->select('sites')->fields()->limit(1)->execute()->fetchAssoc();
+                } else {
+                    self::$isInstalled = false;
+                }
+            } catch (Exception $e) {
+                error_log(
+                    "[IO Exception :: Is Installed Check] (Line ".$e->getLine()." | ".$e->getFile().")  - ".$e->getCode(
+                    )." | ".$e->getMessage()
+                );
+                self::$isInstalled = false; //not installed.
+            }
+        }
+
+        return self::$isInstalled;
+    }
+
+    /**
+     * This function allows for a custom root install path to be specified. It is also possible to provide custom install methods.
+     * @return string
+     */
+    public static function getInstallPath()
+    {
+        return isset(self::$config->app->install->path) && !empty(self::$config->app->install->path) ? self::$config->app->install->path : "install";
+    }
+
+    /**
+     * Add core services to the PHP DI container so that middleware constructors can access them
+     * @param $services
+     * @return Container
+     * @throws Exception
+     */
+    protected function addCoreServices($services)
+    {
+        $containerBuilder = new ContainerBuilder();
+        $containerBuilder->useAttributes(false);
+        $containerBuilder->useAutowiring(true);
+
+        //initialise core services. We array_merge so that these can be overridden by modules if needed.
+        $coreServices = [
+            pathUtility::class => function () {
+                return pathUtility::getInstance();
+            },
+            Singleton::class => function () {
+                return Singleton::getInstance();
+            },
+            Vault::class => function () {
+                return Vault::getInstance();
+            },
+            Mailer::class => function () {
+                return new Mailer();
+            },
+            globalData::class => function () {
+                return globalData::getInstance();
+            },
+            Output::class => function () {
+                return Output::getInstance();
+            },
+        ];
+
+        //Register services declared by modules.
+        $services = array_merge($coreServices, $services);
+        $containerBuilder->addDefinitions($services);
+
+        //Build PHP DI Containers
+        return $containerBuilder->build();
+    }
+
+    private function getInstaller()
+    {
+        return isset(self::$config->app->installer) && !empty(self::$config->app->installer) ? self::$config->app->installer : "\apexl\Io\modules\install\installModule";
+    }
 }
